@@ -1,15 +1,16 @@
-new_board <- function(board, name, cache, ...) {
+new_board <- function(board, name, cache, versions, ...) {
 
   if (is.null(cache)) stop("Please specify the 'cache' parameter.")
 
   board <- structure(list(
       board = board,
       name = name,
-      cache = cache
+      cache = cache,
+      versions = versions
     ),
     class = board)
 
-  board <- board_initialize(board, cache = cache, ...)
+  board <- board_initialize(board, cache = cache, versions = versions, ...)
 
   board
 }
@@ -136,6 +137,7 @@ board_get <- function(name) {
 #' @param board The name of the board to register.
 #' @param name An optional name to identify this board, defaults to the board name.
 #' @param cache The local folder to use as a cache, defaults to \code{board_cache_path()}.
+#' @param versions Should this board be registered with support for versions?
 #' @param ... Additional parameters required to initialize a particular board.
 #'
 #' @details
@@ -147,6 +149,10 @@ board_get <- function(name) {
 #' when using a temp folder, pins will be cleared when your R session restarts. The
 #' cache parameter can be also set with the \code{pins.path} option.
 #'
+#' The \code{versions} parameter default to different values depending on which board
+#' is choosed. For instance, local boards do not use versions by default, but GitHub
+#' boards do.
+#'
 #' @examples
 #' # create a new local board
 #' board_register("local", "other_board", cache = tempfile())
@@ -162,7 +168,11 @@ board_get <- function(name) {
 #'   \code{\link{board_register_datatxt}}.
 #'
 #' @export
-board_register <- function(board, name = board, cache = board_cache_path(), ...) {
+board_register <- function(board,
+                           name = board,
+                           cache = board_cache_path(),
+                           versions = NULL,
+                           ...) {
   params <- list(...)
 
   inferred <- board_infer(board,
@@ -174,7 +184,7 @@ board_register <- function(board, name = board, cache = board_cache_path(), ...)
   params$url <- NULL
 
   new_params <- c(
-    list(inferred$board, inferred$name, cache = cache),
+    list(inferred$board, inferred$name, cache = cache, versions = versions),
     params,
     url = inferred$url
   )
@@ -273,6 +283,8 @@ board_deregister <- function(name, ...) {
 #' # retrieve default board
 #' board_default()
 #'
+#' # revert default board
+#' options(pins.board = NULL)
 #' @export
 board_default <- function() {
   getOption("pins.board", "local")
